library simplified_router;

import 'package:flutter/material.dart';

// TODO SimplifiedRouterMixin

class _Route {
  final String path;

  _Route(this.path);
}

class _RouteState extends ChangeNotifier {
  String _path = '/';

  String get path => _path;

  set path(String s) {
    _path = s;
    notifyListeners();
  }
}

typedef Future<bool> PopRouteCallback();

class SimplifiedRouter {
  final parser = SimplifiedRouteInformationParser();
  final AppRouterDelegate delegate;

  SimplifiedRouter({
    required RouteBuilder builder,
    required GlobalKey<NavigatorState> navigatorKey,
    RouteValidator? shouldNavigateTo,
  }) : delegate = AppRouterDelegate(
            builder: builder,
            navigatorKey: navigatorKey,
            shouldNavigateTo: shouldNavigateTo);

  void navigateTo(String path) {
    delegate._routeState.path = path;
  }
}

typedef Widget RouteBuilder(BuildContext context, String routePath);
typedef Future<bool> RouteValidator(String newPath);

class AppRouterDelegate extends RouterDelegate<_Route>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<_Route> {
  final _routeState = _RouteState();
  final RouteBuilder builder;
  final GlobalKey<NavigatorState> navigatorKey;
  late final RouteValidator shouldNavigateTo;

  AppRouterDelegate({
    required this.builder,
    required this.navigatorKey,
    RouteValidator? shouldNavigateTo,
  }) {
    _routeState.addListener(notifyListeners);
    this.shouldNavigateTo =
        shouldNavigateTo == null ? defaultRouteValidator : shouldNavigateTo;
  }

  Future<bool> defaultRouteValidator(String path) async => true;

  set path(String path) {
    _routeState.path = path;
  }

  @override
  Future<void> setNewRoutePath(_Route path) async {
    if (await shouldNavigateTo(path.path)) {
      _routeState.path = path.path;
    }
  }

  @override
  void dispose() {
    _routeState.dispose();
    super.dispose();
  }

  @override
  _Route get currentConfiguration {
    return _Route(_routeState.path);
  }

  @override
  Widget build(BuildContext context) {
    return builder(context, _routeState.path);
  }
}

class SimplifiedRouteInformationParser extends RouteInformationParser<_Route> {
  @override
  Future<_Route> parseRouteInformation(
      RouteInformation routeInformation) async {
    var uri = Uri.parse(routeInformation.location!);
    return _Route(uri.path);
  }

  @override
  RouteInformation restoreRouteInformation(_Route path) {
    late final location;

    location = path.path;

    return RouteInformation(location: location);
  }
}
